import { field as getFieldAccessor } from "@visactor/vgrammar-util";

import { SemanticMark } from "./semantic-mark";

import { isArray, maxInArray, minInArray } from "@visactor/vutils";

import { GrammarMarkType } from "@visactor/vgrammar-core";

import { PlotMakType } from "./enums";

import { isContinuous } from "@visactor/vscale";

export class RectYSemanticMark extends SemanticMark {
    constructor(id) {
        super(PlotMakType.rectY, id);
    }
    setMarkType() {
        return GrammarMarkType.rect;
    }
    parseScaleByEncode(channel, option) {
        return this.parseScaleOfCommonEncode(channel, option);
    }
    setDefaultCrosshair() {
        return {
            x: {
                crosshairShape: "rect"
            }
        };
    }
    setDefaultTooltip() {
        var _a;
        const encodeY = null === (_a = this.spec.encode) || void 0 === _a ? void 0 : _a.y;
        return {
            content: isArray(encodeY) ? encodeY.map((entry => ({
                value: entry
            }))) : [ {
                value: encodeY
            } ]
        };
    }
    convertMarkEncode(encode) {
        var _a, _b, _c, _d, _e;
        const markEncoder = this.convertSimpleMarkEncode(encode), scaleYId = this.getScaleId("y"), res = {
            x: (datum, el, params) => 0,
            x1: (datum, el, params) => params.viewBox.width()
        };
        if (isArray(null === (_a = markEncoder.y) || void 0 === _a ? void 0 : _a.field)) res.y = {
            field: markEncoder.y.field[0],
            scale: markEncoder.y.scale
        }, res.y1 = {
            field: markEncoder.y.field[1],
            scale: markEncoder.y.scale
        }; else {
            const yAccessor = getFieldAccessor(markEncoder.y.field);
            res.y = (datum, el, params) => {
                const yVals = yAccessor(datum), scale = params[scaleYId];
                return isArray(yVals) ? scale.scale(yVals[0]) : scale.scale(yVals);
            }, res.y1 = (datum, el, params) => {
                var _a, _b, _c, _d;
                const scale = params[scaleYId], yVals = yAccessor(datum);
                if (isArray(yVals) && yVals.length > 1) return scale.scale(yVals[1]);
                if (isContinuous(scale.type)) {
                    const domain = scale.domain(), min = minInArray(domain), max = maxInArray(domain), baseValue = min > 0 ? min : max < 0 ? max : 0;
                    return scale.scale(baseValue);
                }
                return scale.scale(yVals) + (null !== (_d = null !== (_b = null === (_a = scale.bandwidth) || void 0 === _a ? void 0 : _a.call(scale)) && void 0 !== _b ? _b : null === (_c = scale.step) || void 0 === _c ? void 0 : _c.call(scale)) && void 0 !== _d ? _d : 0);
            };
        }
        return markEncoder.stroke && (res.stroke = markEncoder.stroke), markEncoder.color || markEncoder.group ? res.fill = null !== (_b = markEncoder.color) && void 0 !== _b ? _b : markEncoder.group : res.fill = null !== (_d = null === (_c = this.spec.style) || void 0 === _c ? void 0 : _c.fill) && void 0 !== _d ? _d : null === (_e = this.getPalette()) || void 0 === _e ? void 0 : _e[0], 
        res;
    }
}

RectYSemanticMark.type = PlotMakType.rectY;
//# sourceMappingURL=rect-y.js.map